/*--- reply_digest: reply to authentication for outbound registrations ---*/
/*      This is used for register= servers in sip.conf, SIP proxies we register
        with  for receiving calls from.  */
static int reply_digest(struct sip_pvt *p, struct sip_request *req,
	char *header, int sipmethod,  char *digest, int digest_len)
{
	char tmp[512] = "";
	char *c;

	/* table of recognised keywords, and places where they should be copied */
	struct x {
		const char *key;
		char *dst;
		int dstlen;
	} *i, keys[] = {
		{ "realm=", p->realm, sizeof(p->realm) },
		{ "nonce=", p->nonce, sizeof(p->nonce) },
		{ "opaque=", p->opaque, sizeof(p->opaque) },
		{ "qop=", p->qop, sizeof(p->qop) },
		{ "domain=", p->domain, sizeof(p->domain) },
		{ NULL, NULL, 0 },
	};

	ast_copy_string(tmp, get_header(req, header),sizeof(tmp));
	if (ast_strlen_zero(tmp)) 
		return -1;
	if (strstr(tmp, "Digest ") != tmp) {
		ast_log(LOG_WARNING, "missing Digest.\n");
		return -1;
	}
	c = tmp + strlen("Digest ");
	for (i = keys; i->key != NULL; i++)
		i->dst[0] = '\0';	/* init all to empty strings */
	for (; c && *(c = skip_blanks(c)) ; c++) {	/* lookup for keys */
		for (i = keys; i->key != NULL; i++) {
			char *src;
			if (strncasecmp(c, i->key, strlen(i->key)) != 0)
				continue;
			c += strlen(i->key);
			if ((*c == '\"')) {
				src = ++c;
				if ((c = strchr(c,'\"')))
					*c = '\0';
			} else {
				src = c;
				if ((c = strchr(c,',')))
					*c = '\0';
			}
			ast_copy_string(i->dst, src, i->dstlen);
			break;
		}
		if (i->key == NULL)
			c = strchr(c,',');
	}
	if (strlen(tmp) >= sizeof(tmp))
		ast_log(LOG_WARNING, "Buffer overflow detected!  Please file a bug.\n");

	/* copy realm and nonce for later authorization of CANCELs and BYEs */
	if (p->registry) {
		struct sip_registry *r = p->registry;

		ast_copy_string(r->realm, p->realm, sizeof(r->realm));
		ast_copy_string(r->nonce, p->nonce, sizeof(r->nonce));
		ast_copy_string(r->domain, p->domain, sizeof(r->domain));
		ast_copy_string(r->opaque, p->opaque, sizeof(r->opaque));
		ast_copy_string(r->qop, p->qop, sizeof(r->qop));
	}
	build_reply_digest(p, sipmethod, digest, digest_len); 
	return 0;
}
